//+------------+-----------------------------------------------------+
//| v.11.09.06 |                                  SnakeInBorders.mq4 |
//+------------+                                   " !" |
//|    |              Bookkeeper, 2006, yuzefovich@gmail.com |
//+------------------------------------------------------------------+
//|  Snake    ,  |
//|    BorderTop[]  BorderBot[],   Mart[]. |
//|                                                                  |
//|   :                                             |
//|  SnakeRange -    Snake' Axis[] -  |
//|  = 4;                                                            |
//|  FilterPeriod -  . (   13);      |
//|  MartFiltr -   .  MartFiltr , |
//|      .          |
//|  ,   = 2;                                    |
//|   Snake'  : HardCalc = true -  ,      |
//|  = false -   Axis[Pos]...[Pos+SnakeRange+1].|
//|    Mart[]  :                    |
//|      -            |
//|  ,       .     |
//|  ,    , -    . |
//|    ,     ,   |
//|   ""   .      |
//|       .   |
//|      .     |
//|            |
//|   ,     -     |
//|    .                                     |
//|  Snake    ,   |
//|      ,          |
//|   ,  Mart[]   .       |
//|    MA, OA, AC,..  HardCalc = true,   |
//|     ZigZag, Channel,.. - HardCalc = false,    |
//|   MartFiltr   3...5.     | 
//|     :   ,          | 
//|   Mart[]     .                |
//+------------------------------------------------------------------+
#property copyright ""
#property link      "yuzefovich@gmail.com"
//----
#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  Red
#property indicator_color2  Red
#property indicator_color3  Blue
//----
extern int    SnakeRange   = 2; 
extern int    FilterPeriod = 24; 
extern double MartFiltr    = 2;
extern bool   HardCalc     = true;
//----
double BorderTop[];
double BorderBot[];
double Mart[];
double Axis[];
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void deinit()
  {
   Comment("");
   return;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int init()
  {
   int draw_begin;
   draw_begin = FilterPeriod + SnakeRange + 2;
   IndicatorBuffers  (4);
   SetIndexBuffer    (0, BorderTop);
   SetIndexBuffer    (1, BorderBot); 
   SetIndexBuffer    (2, Mart); 
   SetIndexBuffer    (3, Axis); 
   SetIndexStyle     (0, DRAW_LINE);
   SetIndexStyle     (1, DRAW_LINE);
   SetIndexStyle     (2, DRAW_LINE, 0, 2); 
   SetIndexDrawBegin (0, draw_begin);
   SetIndexDrawBegin (1, draw_begin);
   SetIndexDrawBegin (2, draw_begin);
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int counted_bars = IndicatorCounted();
   int limit, i;
   if(counted_bars < 0)  
       return(-1);
   if(Bars <= (FilterPeriod + SnakeRange+2)) 
       return(0);
   if(SnakeRange < 4) 
       SnakeRange = 4;
   if(counted_bars > 0)  
       counted_bars--;
   limit = Bars - counted_bars;
   for(i = limit; i >= 0; i--) 
       MainCalculation(i);
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void MainCalculation(int Pos)
  {
   int v;
   MartAxis(Pos);
   if(HardCalc == true) 
       SmoothOverMart2(Pos);
   else 
       SmoothOverMart(Pos);
//----
   return;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void MartAxis(int Pos)
  {  
   int    SnakeWeight, i, w, ww, Shift;
   double SnakeSum;
   Axis[Pos] = iMA(NULL, 0, SnakeRange + 1, 0, MODE_LWMA, PRICE_WEIGHTED, Pos);
   for(Shift = Pos + SnakeRange + 2; Shift > Pos; Shift--)
     {
       SnakeSum = 0.0;
       SnakeWeight = 0;
       i = 0;
       w = Shift + SnakeRange;
       ww = Shift - SnakeRange;
       if(ww < Pos) 
           ww = Pos;
       while(w >= Shift)
         {
           i++;
           SnakeSum = SnakeSum + i*SnakePrice(w);
           SnakeWeight = SnakeWeight + i;
           w--;
         }
       while(w >= ww)
         {
           i--;
           SnakeSum = SnakeSum + i*SnakePrice(w);
           SnakeWeight = SnakeWeight + i;
           w--;
         }
       Axis[Shift] = SnakeSum / SnakeWeight;
     }
   return;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double SnakePrice(int Shift)
  {
   return((2*Close[Shift] + High[Shift] + Low[Shift]) / 4);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void SmoothOverMart(int Pos)
  { //     Pos...(Pos+SnakeRange+2) 
   int    Shift;
   double a, t, b;
   for(Shift = Pos + SnakeRange + 2; Shift >= Pos; Shift--)
     {
       t = Axis[ArrayMaximum(Axis, FilterPeriod, Shift)];
       b = Axis[ArrayMinimum(Axis, FilterPeriod, Shift)];
       a = Axis[Shift];
       BorderTop[Shift] = (2*(1 + MartFiltr)*a + (t - b)) / 2 / (1 + MartFiltr);
       BorderBot[Shift] = (2*(1 + MartFiltr)*a - (t - b)) / 2 / (1 + MartFiltr);
       Mart[Shift] = (2*(2 + MartFiltr)*a - (t + b)) / 2 / (1 + MartFiltr);
     }
   return;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void SmoothOverMart2(int Shift)
  { //      
   double a, t, b;
   t = Axis[ArrayMaximum(Axis, FilterPeriod, Shift)];
   b = Axis[ArrayMinimum(Axis, FilterPeriod, Shift)];
   a = Axis[Shift];
   BorderTop[Shift] = (2*(1 + MartFiltr)*a + (t - b))/2/(1 + MartFiltr);
   BorderBot[Shift] = (2*(1 + MartFiltr)*a - (t - b))/2/(1 + MartFiltr);
   Mart[Shift] = (2*(2 + MartFiltr)*a - (t + b)) / 2 / (1 + MartFiltr);
   return;
  }
//---------------------------------------------------------------------

